@@ -70,11 +70,13 @@ class flexrf_base(db_base.db_base):
         self.spi_format = usrp1.SPI_FMT_MSB | usrp1.SPI_FMT_HDR_0
 
         self._u._write_oe(self._which, 0, 0xffff)   # turn off all outputs
-        self.set_refclk(False)                      # disable refclk
+        self._enable_refclk(False)                      # disable refclk
 
         g = self.gain_range()                       # initialize gain
         self.set_gain(float(g[0]+g[1]) / 2)
 
+        self.set_auto_tr(False)
+        
         if debug_using_gui:
             title = "FlexRF Debug Rx"
             if self._tx:
@@ -85,25 +87,9 @@ class flexrf_base(db_base.db_base):
 
     def __del__(self):
         #print "flexrf_base.__del__"
-        self._u.write_io(self._which, 0, POWER_UP)   # turn off power to board
-        self.set_refclk(False)                       # turn off refclk
-        self.set_atr_mask(0)                         # ensure ATR enable doesn't effect us
-
-    def set_refclk(self, on):
-        """
-        Enable or disable the sending of the reference clock.
-        If enabled, the reference clock is sent on daughterboard i/o pin 0.
-        @param on: True to enable reference clock
-        """
-        if on:
-            self._u._write_oe(self._which, CLOCK_OUT, CLOCK_OUT)
-            self._u._write_fpga_reg(FR_TX_A_REFCLK + self._slot,
-                                    ((self._refclk_divisor() & bmFR_REFCLK_DIVISOR_MASK)
-                                     | bmFR_REFCLK_EN))
-        else:
-            self._u._write_oe(self._which, 0, CLOCK_OUT)
-            self._u._write_fpga_reg(FR_TX_A_REFCLK + self._slot, 0)
-            
+        self._u.write_io(self._which, self.power_off, POWER_UP)   # turn off power to board
+        self._enable_refclk(False)                       # turn off refclk
+        self.set_auto_tr(False)
 
     def _write_all(self, R, control, N):
         """
@@ -167,18 +153,7 @@ class flexrf_base(db_base.db_base):
         """
         raise NotImplementedError
 
-    def _refclk_divisor(self):
-        """
-        Return value to stick in REFCLK_DIVISOR register
-
-        Override this in derived classes.
-        """
-        raise NotImplementedError
-
     def _refclk_freq(self):
-        # FIXME this needs work if we're using the FPGA generated refclk.
-        # As of now we're using the oscillators on the d'board.
-
         # return float(self._u.fpga_master_clock_freq())/self._refclk_divisor()
         return 64e6/self._refclk_divisor()
 
@@ -194,8 +169,7 @@ class flexrf_base(db_base.db_base):
         # FPGA's DC removal loop's time constant.  We were seeing a
         # problem when running with discontinuous transmission.
         # Offsetting the LO made the problem go away.
-        lo_offset = 4e6             # FIXME may want to be a function of d'board
-        freq += lo_offset
+        freq += self.lo_offset
         
         R, control, N, actual_freq = self._compute_regs(freq)
         #print R, control,N,actual_freq
@@ -245,27 +219,29 @@ class flexrf_base_tx(flexrf_base):
         @param which: 0 or 1 corresponding to side TX_A or TX_B respectively.
         """
         flexrf_base.__init__(self, usrp, which)
-
         self.spi_enable = (SPI_ENABLE_TX_A, SPI_ENABLE_TX_B)[which]
-        
-        self.set_auto_tr(False)
 
         # power up the transmit side, but don't enable the mixer
         self._u._write_oe(self._which,(POWER_UP|RX_TXN|ENABLE), 0xffff)
-        self._u.write_io(self._which, (POWER_UP|RX_TXN), (POWER_UP|RX_TXN|ENABLE))
-
-        # setup for Auto T/R switching in case it's enabled later
-        self.set_atr_mask (RX_TXN | ENABLE)
-        self.set_atr_txval(0      | ENABLE)
-        self.set_atr_rxval(RX_TXN | 0)
+        self._u.write_io(self._which, (self.power_on|RX_TXN), (POWER_UP|RX_TXN|ENABLE))
+        self.lo_offset = 4e6             # FIXME may want to be a function of d'board
 
     def __del__(self):
         #print "flexrf_base_tx.__del__"
-        self.set_auto_tr(False)
         # Power down and leave the T/R switch in the R position
-        self._u.write_io(self._which, (RX_TXN), (POWER_UP|RX_TXN|ENABLE))
+        self._u.write_io(self._which, (self.power_off|RX_TXN), (POWER_UP|RX_TXN|ENABLE))
         flexrf_base.__del__(self)
 
+    def set_auto_tr(self, on):
+        if on:
+            self.set_atr_mask (RX_TXN | ENABLE)
+            self.set_atr_txval(0      | ENABLE)
+            self.set_atr_rxval(RX_TXN | 0)
+        else:
+            self.set_atr_mask (0)
+            self.set_atr_txval(0)
+            self.set_atr_rxval(0)
+
     def set_enable(self, on):
         """
         Enable transmitter if on is True
@@ -277,7 +253,6 @@ class flexrf_base_tx(flexrf_base):
             v = RX_TXN
         self._u.write_io(self._which, v, mask)
 
-            
     def gain_range(self):
         """
         Return range of gain that can be set by this d'board.
@@ -307,33 +282,34 @@ class flexrf_base_rx(flexrf_base):
         @param which: 0 or 1 corresponding to side RX_A or RX_B respectively.
         """
         flexrf_base.__init__(self, usrp, which)
-
         self.spi_enable = (SPI_ENABLE_RX_A, SPI_ENABLE_RX_B)[which]
         
         self._u._write_oe(self._which, (POWER_UP|RX2_RX1N|ENABLE), 0xffff)
-        self._u.write_io(self._which,  (POWER_UP|RX2_RX1N|ENABLE), (POWER_UP|RX2_RX1N|ENABLE))
+        self._u.write_io(self._which,  (self.power_on|RX2_RX1N|ENABLE), (POWER_UP|RX2_RX1N|ENABLE))
 
         # set up for RX on TX/RX port
         self.select_rx_antenna('TX/RX')
 
-        if which==0:
-            self._u.set_adc_buffer_bypass(0, True)
-            self._u.set_adc_buffer_bypass(1, True)
-        else:
-            self._u.set_adc_buffer_bypass(2, True)
-            self._u.set_adc_buffer_bypass(3, True)
+        self.bypass_adc_buffers(True)
 
-        # setup for Auto T/R switching in case it's enabled later
-        self.set_atr_mask (ENABLE)
-        self.set_atr_txval(     0)
-        self.set_atr_rxval(ENABLE)
+        self.lo_offset = -4e6             # FIXME may want to be a function of d'board
 
     def __del__(self):
         # print "flexrf_base_rx.__del__"
         # Power down
-        self._u.write_io(self._which, 0, (POWER_UP|ENABLE))
+        self._u.write_io(self._which, self.power_off, (POWER_UP|ENABLE))
         flexrf_base.__del__(self)
     
+    def set_auto_tr(self, on):
+        if on:
+            self.set_atr_mask (ENABLE)
+            self.set_atr_txval(     0)
+            self.set_atr_rxval(ENABLE)
+        else:
+            self.set_atr_mask (0)
+            self.set_atr_txval(0)
+            self.set_atr_rxval(0)
+
     def select_rx_antenna(self, which_antenna):
         """
         Specify which antenna port to use for reception.
@@ -345,7 +321,31 @@ class flexrf_base_rx(flexrf_base):
             self._u.write_io(self._which, RX2_RX1N, RX2_RX1N)
         else:
             raise ValueError, "which_antenna must be either 'TX/RX' or 'RX2'"
-            
+
+    def set_gain(self, gain):
+        """
+        Set the gain.
+
+        @param gain:  gain in decibels
+        @returns True/False
+        """
+        maxgain = self.gain_range()[1] - self._u.pga_max()
+        if gain > maxgain:
+            pga_gain = gain-maxgain
+            assert pga_gain <= self._u.pga_max()
+            agc_gain = maxgain
+        else:
+            pga_gain = 0
+            agc_gain = gain
+        V_maxgain = .2
+        V_mingain = 1.2
+        V_fullscale = 3.3
+        dac_value = (agc_gain*(V_maxgain-V_mingain)/maxgain + V_mingain)*4096/V_fullscale
+        assert dac_value>=0 and dac_value<4096
+        return self._u.write_aux_dac(self._which, 0, int(dac_value)) and \
+               self._set_pga(int(pga_gain))
+
+
 # ----------------------------------------------------------------
 
 class _AD4360_common(object):
@@ -406,8 +406,6 @@ class _AD4360_common(object):
         """
         Return value to stick in REFCLK_DIVISOR register
         """
-        # send it 16 MHz (Div by 4)
-        #return 4
         return 1
     
     def _prescaler(self):
@@ -437,8 +435,8 @@ class _2400_common(_AD4360_common):
         self.CPGAIN = 0   # bit 21
         self.freq_mult = 1
 
-    #def freq_range(self):           # FIXME
-    #    return (x, y, z)
+    def freq_range(self):           # FIXME
+        return (2300e6, 2500e6, 4e6)
 
 #----------------------------------------------------------------------
 class _1200_common(_AD4360_common):
@@ -490,9 +488,7 @@ class _400_common(_AD4360_common):
         _AD4360_common.__init__(self)
 
         # Band-specific R-Register Values
-        #self.R_DIV = 2  # bits 15:2 
-        self.R_DIV = 32  # bits 15:2 
-
+        self.R_DIV = 16  # bits 15:2 
    
         # Band-specific C-Register values
         self.P = 0        # bits 23,22   Div by 8/9
@@ -517,81 +513,72 @@ class _400_common(_AD4360_common):
 #------------------------------------------------------------    
 class db_flexrf_2400_tx(_2400_common, flexrf_base_tx):
     def __init__(self, usrp, which):
-        self._pga_gain_value = 0                      # FIXME, set correct value
+        self.power_on = ~POWER_UP
+        self.power_off = ~POWER_UP    # powering it off kills the serial bus
         flexrf_base_tx.__init__(self, usrp, which)
         _2400_common.__init__(self)
         
 class db_flexrf_2400_rx(_2400_common, flexrf_base_rx):
     def __init__(self, usrp, which):
-        self._pga_gain_value = 0                      # FIXME, set correct value
+        self.power_on = ~POWER_UP
+        self.power_off = ~POWER_UP   # Powering it off kills the serial bus
         flexrf_base_rx.__init__(self, usrp, which)
         _2400_common.__init__(self)
 
+    def gain_range(self):
+        """
+        Return range of gain that can be set by this d'board.
+        
+        @returns (min_gain, max_gain, step_size)
+        Where gains are expressed in decibels (your mileage may vary)
+        """
+        return (self._u.pga_min(), self._u.pga_max() + 70, 0.05)
+
+    def i_and_q_swapped(self):
+        return True
+
 class db_flexrf_1200_tx(_1200_common, flexrf_base_tx):
     def __init__(self, usrp, which):
-        self._pga_gain_value = 0                      # FIXME, set correct value
         flexrf_base_tx.__init__(self, usrp, which)
         _1200_common.__init__(self)
         
 class db_flexrf_1200_rx(_1200_common, flexrf_base_rx):
     def __init__(self, usrp, which):
-        self._pga_gain_value = 0                      # FIXME, set correct value
         flexrf_base_rx.__init__(self, usrp, which)
         _1200_common.__init__(self)
 
 class db_flexrf_900_tx(_900_common, flexrf_base_tx):
     def __init__(self, usrp, which):
-        self._pga_gain_value = 0                      # FIXME, set correct value
         flexrf_base_tx.__init__(self, usrp, which)
         _900_common.__init__(self)
         
 class db_flexrf_900_rx(_900_common, flexrf_base_rx):
     def __init__(self, usrp, which):
-        self._pga_gain_value = 0                      # FIXME, set correct value
         flexrf_base_rx.__init__(self, usrp, which)
         _900_common.__init__(self)
 
 class db_flexrf_400_tx(_400_common, flexrf_base_tx):
     def __init__(self, usrp, which):
-        self._pga_gain_value = 0                      # FIXME, set correct value
+        self.power_on = POWER_UP
+        self.power_off = ~POWER_UP
         flexrf_base_tx.__init__(self, usrp, which)
         _400_common.__init__(self)
         
 class db_flexrf_400_rx(_400_common, flexrf_base_rx):
     def __init__(self, usrp, which):
-        self._pga_gain_value = 0                      # FIXME, set correct value
+        self.power_on = POWER_UP
+        self.power_off = ~POWER_UP
         flexrf_base_rx.__init__(self, usrp, which)
         _400_common.__init__(self)
 
     def gain_range(self):
         """
         Return range of gain that can be set by this d'board.
-
+        
         @returns (min_gain, max_gain, step_size)
         Where gains are expressed in decibels (your mileage may vary)
         """
-        return (self._u.pga_min(), self._u.pga_max() + 45, self._u.pga_db_per_step())
-
-    def set_gain(self, gain):
-        """
-        Set the gain.
-
-        @param gain:  gain in decibels
-        @returns True/False
-        """
-        if gain > 45:
-            pga_gain = gain-45
-            assert pga_gain <= self._u.pga_max()
-            agc_gain = 45
-        else:
-            pga_gain = 0
-            agc_gain = gain
-        V_maxgain = .2
-        V_mingain = 1.2
-        V_fullscale = 3.3
-        dac_value = (agc_gain*(V_maxgain-V_mingain)/45 + V_mingain)*4096/V_fullscale
-        assert dac_value>=0 and dac_value<4096
-        return self._u.write_aux_dac(self._which, 0, int(dac_value)) and self._set_pga(int(gain))
+        return (self._u.pga_min(), self._u.pga_max() + 45, 0.035)
 
     def i_and_q_swapped(self):
         return True
