@@ -25,6 +25,7 @@ import usrp_prims
 import usrp_dbid
 from gnuradio import usrp1              # usrp Rev 1 and later
 from gnuradio import gru
+from usrp_fpga_regs import *
 
 FPGA_MODE_NORMAL   = usrp1.FPGA_MODE_NORMAL
 FPGA_MODE_LOOPBACK = usrp1.FPGA_MODE_LOOPBACK
@@ -56,6 +57,7 @@ import db_instantiator
 import db_basic
 import db_dbs_rx
 import db_flexrf
+import db_flexrf_mimo
 import db_tv_rx
 
 
@@ -69,20 +71,16 @@ def _look_for_usrp(which):
     @return: Returns version number, or raises RuntimeError
     @rtype: int
     """
-    d = usrp_prims.usrp_find_device (which)
+    d = usrp_prims.usrp_find_device(which)
     if not d:
         raise RuntimeError, "Unable to find USRP #%d" % (which,)
 
-    if usrp_prims.usrp_usrp0_p (d):
-        return 0
-    elif usrp_prims.usrp_usrp1_p (d) or usrp_prims.usrp_usrp2_p (d):
-        return 1
-    else:
-        raise RuntimeError, "Hmmm, unknown usrp rev"
+    return usrp_prims.usrp_hw_rev(d)
 
-def _ensure_rev1(which):
+
+def _ensure_rev2(which):
     v = _look_for_usrp(which)
-    if v != 1:
+    if not v in (2, 4):
         raise RuntimeError, "Sorry, unsupported USRP revision (rev=%d)" % (v,)
 
 
@@ -127,7 +125,10 @@ def tune(u, chan, subdev, target_freq):
     # If the spectrum is inverted, and the daughterboard doesn't do
     # quadrature downconversion, we can fix the inversion by flipping the
     # sign of the dxc_freq...  (This only happens using the basic_rx board)
-        
+
+    if subdev.spectrum_inverted():
+        inverted = not(inverted)
+    
     if inverted and not(subdev.is_quadrature()):
         dxc_freq = -dxc_freq
         inverted = not(inverted)
@@ -157,77 +158,113 @@ def tune(u, chan, subdev, target_freq):
 # [Also provides keyword args.]
 # ------------------------------------------------------------------------
 
-class sink_c(object):
-    def __init__(self, which=0, interp_rate=128, nchan=1, mux=0x98):
-        _ensure_rev1(which)
-        self._u = usrp1.sink_c(which, interp_rate, nchan, mux)
-        # Add the db attribute, which contains a 2-tuple of tuples of daughterboard classes
-        self.db = (db_instantiator.instantiate(self._u, 0),
-                   db_instantiator.instantiate(self._u, 1))
+class usrp_common(object):
+    def __init__(self):
+        # read capability register
+        r = self._u._read_fpga_reg(FR_RB_CAPS)
+        if r < 0:
+            r += 2**32
+        if r == 0xaa55ff77:    # value of this reg prior to being defined as cap reg
+            r = ((2 << bmFR_RB_CAPS_NDUC_SHIFT)
+                 | (2 << bmFR_RB_CAPS_NDDC_SHIFT)
+                 | bmFR_RB_CAPS_RX_HAS_HALFBAND)
+        self._fpga_caps = r
 
-    def __del__(self):
-        self.db = None          # will fire d'board destructors
-        self._u = None          # will fire usrp1.* destructor
-        
+        if False:
+            print "FR_RB_CAPS      = %#08x" % (self._fpga_caps,)
+            print "has_rx_halfband =", self.has_rx_halfband()
+            print "nDDCs           =", self.nddc()
+            print "has_tx_halfband =", self.has_tx_halfband()
+            print "nDUCs           =", self.nduc()
+    
     def __getattr__(self, name):
         return getattr(self._u, name)
 
     def tune(self, chan, subdev, target_freq):
         return tune(self, chan, subdev, target_freq)
 
-class sink_s(object):
-    def __init__(self, which=0, interp_rate=128, nchan=1, mux=0x98):
-        _ensure_rev1(which)
-        self._u = usrp1.sink_s(which, interp_rate, nchan, mux)
+    def has_rx_halfband(self):
+        return self._fpga_caps & bmFR_RB_CAPS_RX_HAS_HALFBAND != 0
+
+    def has_tx_halfband(self):
+        return self._fpga_caps & bmFR_RB_CAPS_TX_HAS_HALFBAND != 0
+
+    def nddc(self):
+        """
+        Number of Digital Down Converters implemented in FPGA
+        """
+        return (self._fpga_caps & bmFR_RB_CAPS_NDDC_MASK) >> bmFR_RB_CAPS_NDDC_SHIFT
+
+    def nduc(self):
+        """
+        Number of Digital Up Converters implemented in FPGA
+        """
+        return (self._fpga_caps & bmFR_RB_CAPS_NDUC_MASK) >> bmFR_RB_CAPS_NDUC_SHIFT
+
+
+class sink_c(usrp_common):
+    def __init__(self, which=0, interp_rate=128, nchan=1, mux=0x98,
+                 fpga_filename="", firmware_filename=""):
+        _ensure_rev2(which)
+        self._u = usrp1.sink_c(which, interp_rate, nchan, mux,
+                               fpga_filename, firmware_filename)
         # Add the db attribute, which contains a 2-tuple of tuples of daughterboard classes
         self.db = (db_instantiator.instantiate(self._u, 0),
                    db_instantiator.instantiate(self._u, 1))
+        usrp_common.__init__(self)
+
+    def __del__(self):
+        self.db = None          # will fire d'board destructors
+        self._u = None          # will fire usrp1.* destructor
+
+
+class sink_s(usrp_common):
+    def __init__(self, which=0, interp_rate=128, nchan=1, mux=0x98,
+                 fpga_filename="", firmware_filename=""):
+        _ensure_rev2(which)
+        self._u = usrp1.sink_s(which, interp_rate, nchan, mux,
+                               fpga_filename, firmware_filename)
+        # Add the db attribute, which contains a 2-tuple of tuples of daughterboard classes
+        self.db = (db_instantiator.instantiate(self._u, 0),
+                   db_instantiator.instantiate(self._u, 1))
+        usrp_common.__init__(self)
 
     def __del__(self):
         self.db = None          # will fire d'board destructors
         self._u = None          # will fire usrp1.* destructor
         
-    def __getattr__(self, name):
-        return getattr(self._u, name)
 
-    def tune(self, chan, subdev, target_freq):
-        return tune(self, chan, subdev, target_freq)
-
-class source_c(object):
-    def __init__(self, which=0, decim_rate=64, nchan=1, mux=0x32103210, mode=0):
-        _ensure_rev1(which)
-        self._u = usrp1.source_c(which, decim_rate, nchan, mux, mode)
+class source_c(usrp_common):
+    def __init__(self, which=0, decim_rate=64, nchan=1, mux=0x32103210, mode=0,
+                 fpga_filename="", firmware_filename=""):
+        _ensure_rev2(which)
+        self._u = usrp1.source_c(which, decim_rate, nchan, mux, mode,
+                                 fpga_filename, firmware_filename)
         # Add the db attribute, which contains a 2-tuple of tuples of daughterboard classes
         self.db = (db_instantiator.instantiate(self._u, 0),
                    db_instantiator.instantiate(self._u, 1))
+        usrp_common.__init__(self)
+
+    def __del__(self):
+        self.db = None          # will fire d'board destructors
+        self._u = None          # will fire usrp1.* destructor
+
+
+class source_s(usrp_common):
+    def __init__(self, which=0, decim_rate=64, nchan=1, mux=0x32103210, mode=0,
+                 fpga_filename="", firmware_filename=""):
+        _ensure_rev2(which)
+        self._u = usrp1.source_s(which, decim_rate, nchan, mux, mode,
+                                 fpga_filename, firmware_filename)
+        # Add the db attribute, which contains a 2-tuple of tuples of daughterboard classes
+        self.db = (db_instantiator.instantiate(self._u, 0),
+                   db_instantiator.instantiate(self._u, 1))
+        usrp_common.__init__(self)
 
     def __del__(self):
         self.db = None          # will fire d'board destructors
         self._u = None          # will fire usrp1.* destructor
         
-    def __getattr__(self, name):
-        return getattr(self._u, name)
-
-    def tune(self, chan, subdev, target_freq):
-        return tune(self, chan, subdev, target_freq)
-
-class source_s(object):
-    def __init__(self, which=0, decim_rate=64, nchan=1, mux=0x32103210, mode=0):
-        _ensure_rev1(which)
-        self._u = usrp1.source_s(which, decim_rate, nchan, mux, mode)
-        # Add the db attribute, which contains a 2-tuple of tuples of daughterboard classes
-        self.db = (db_instantiator.instantiate(self._u, 0),
-                   db_instantiator.instantiate(self._u, 1))
-
-    def __del__(self):
-        self.db = None          # will fire d'board destructors
-        self._u = None          # will fire usrp1.* destructor
-        
-    def __getattr__(self, name):
-        return getattr(self._u, name)
-
-    def tune(self, chan, subdev, target_freq):
-        return tune(self, chan, subdev, target_freq)
 
 # ------------------------------------------------------------------------
 #                               utilities
@@ -306,6 +343,27 @@ def determine_rx_mux_value(u, subdev_spec):
     return gru.hexint(truth_table[(side, uses, swap_iq)])
 
 
+def determine_tx_mux_value(u, subdev_spec):
+    """
+    Determine appropriate Tx mux value as a function of the subdevice choosen.
+
+    @param u:           instance of USRP source
+    @param subdev_spec: return value from subdev option parser.  
+    @type  subdev_spec: (side, subdev), where side is 0 or 1 and subdev is 0
+    @returns:           the Rx mux value
+    """
+    # This is simpler than the rx case.  Either you want to talk
+    # to side A or side B.  If you want to talk to both sides at once,
+    # determine the value manually.
+
+    side = subdev_spec[0]  # side A = 0, side B = 1
+
+    if not(side in (0, 1)):
+        raise ValueError, "Invalid subdev_spec: %r:" % (subdev_spec,)
+
+    return gru.hexint([0x0098, 0x9800][side])
+
+
 def selected_subdev(u, subdev_spec):
     """
     Return the user specified daughterboard subdevice.
@@ -384,7 +442,9 @@ def pick_rx_subdevice(u):
                            usrp_dbid.FLEX_1200_RX,
                            usrp_dbid.FLEX_2400_RX,
                            usrp_dbid.TV_RX,
+                           usrp_dbid.TV_RX_REV_2,
                            usrp_dbid.DBS_RX,
+                           usrp_dbid.DBS_RX_REV_2_1,
                            usrp_dbid.BASIC_RX))
 
 def pick_subdev(u, candidates):
