@@ -1,18 +1,18 @@
-# This file was created automatically by SWIG 1.3.27.
+# This file was created automatically by SWIG 1.3.29.
 # Don't modify this file, modify the SWIG interface instead.
 
 import _usrp1
-
+import new
+new_instancemethod = new.instancemethod
 def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
+    if (name == "thisown"): return self.this.own(value)
     if (name == "this"):
-        if isinstance(value, class_type):
-            self.__dict__[name] = value.this
-            if hasattr(value,"thisown"): self.__dict__["thisown"] = value.thisown
-            del value.thisown
+        if type(value).__name__ == 'PySwigObject':
+            self.__dict__[name] = value
             return
     method = class_type.__swig_setmethods__.get(name,None)
     if method: return method(self,value)
-    if (not static) or hasattr(self,name) or (name == "thisown"):
+    if (not static) or hasattr(self,name):
         self.__dict__[name] = value
     else:
         raise AttributeError("You cannot add attributes to %s" % self)
@@ -21,10 +21,16 @@ def _swig_setattr(self,class_type,name,value):
     return _swig_setattr_nondynamic(self,class_type,name,value,0)
 
 def _swig_getattr(self,class_type,name):
+    if (name == "thisown"): return self.this.own()
     method = class_type.__swig_getmethods__.get(name,None)
     if method: return method(self)
     raise AttributeError,name
 
+def _swig_repr(self):
+    try: strthis = "proxy of " + self.this.__repr__()
+    except: strthis = ""
+    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)
+
 import types
 try:
     _object = types.ObjectType
@@ -37,7 +43,8 @@ del types
 
 def _swig_setattr_nondynamic_method(set):
     def set_attr(self,name,value):
-        if hasattr(self,name) or (name in ("this", "thisown")):
+        if (name == "thisown"): return self.this.own(value)
+        if hasattr(self,name) or (name == "this"):
             set(self,name,value)
         else:
             raise AttributeError("You cannot add attributes to %s" % self)
@@ -64,15 +71,11 @@ FPGA_MODE_LOOPBACK = _usrp1.FPGA_MODE_LOOPBACK
 FPGA_MODE_COUNTING = _usrp1.FPGA_MODE_COUNTING
 class usrp1_sink_base(object):
     """Proxy of C++ usrp1_sink_base class"""
-    def __init__(self): raise RuntimeError, "No constructor defined"
-    def __repr__(self):
-        return "<%s.%s; proxy of C++ usrp1_sink_base instance at %s>" % (self.__class__.__module__, self.__class__.__name__, self.this,)
-    def __del__(self, destroy=_usrp1.delete_usrp1_sink_base):
-        """__del__(self)"""
-        try:
-            if self.thisown: destroy(self)
-        except: pass
-
+    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
+    def __init__(self): raise AttributeError, "No constructor defined"
+    __repr__ = _swig_repr
+    __swig_destroy__ = _usrp1.delete_usrp1_sink_base
+    __del__ = lambda self : None;
     def set_interp_rate(*args):
         """set_interp_rate(self, unsigned int rate) -> bool"""
         return _usrp1.usrp1_sink_base_set_interp_rate(*args)
@@ -157,6 +160,10 @@ class usrp1_sink_base(object):
         """set_adc_buffer_bypass(self, int which, bool bypass) -> bool"""
         return _usrp1.usrp1_sink_base_set_adc_buffer_bypass(*args)
 
+    def serial_number(*args):
+        """serial_number(self) -> string"""
+        return _usrp1.usrp1_sink_base_serial_number(*args)
+
     def _write_oe(*args):
         """_write_oe(self, int which_dboard, int value, int mask) -> bool"""
         return _usrp1.usrp1_sink_base__write_oe(*args)
@@ -217,25 +224,16 @@ class usrp1_sink_base(object):
         """_read_spi(self, int optional_header, int enables, int format, int len) -> string"""
         return _usrp1.usrp1_sink_base__read_spi(*args)
 
-
-class usrp1_sink_basePtr(usrp1_sink_base):
-    def __init__(self, this):
-        self.this = this
-        if not hasattr(self,"thisown"): self.thisown = 0
-        self.__class__ = usrp1_sink_base
-_usrp1.usrp1_sink_base_swigregister(usrp1_sink_basePtr)
+usrp1_sink_base_swigregister = _usrp1.usrp1_sink_base_swigregister
+usrp1_sink_base_swigregister(usrp1_sink_base)
 
 class usrp1_source_base(object):
     """Proxy of C++ usrp1_source_base class"""
-    def __init__(self): raise RuntimeError, "No constructor defined"
-    def __repr__(self):
-        return "<%s.%s; proxy of C++ usrp1_source_base instance at %s>" % (self.__class__.__module__, self.__class__.__name__, self.this,)
-    def __del__(self, destroy=_usrp1.delete_usrp1_source_base):
-        """__del__(self)"""
-        try:
-            if self.thisown: destroy(self)
-        except: pass
-
+    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
+    def __init__(self): raise AttributeError, "No constructor defined"
+    __repr__ = _swig_repr
+    __swig_destroy__ = _usrp1.delete_usrp1_source_base
+    __del__ = lambda self : None;
     def set_decim_rate(*args):
         """set_decim_rate(self, unsigned int rate) -> bool"""
         return _usrp1.usrp1_source_base_set_decim_rate(*args)
@@ -328,6 +326,10 @@ class usrp1_source_base(object):
         """set_adc_buffer_bypass(self, int which, bool bypass) -> bool"""
         return _usrp1.usrp1_source_base_set_adc_buffer_bypass(*args)
 
+    def serial_number(*args):
+        """serial_number(self) -> string"""
+        return _usrp1.usrp1_source_base_serial_number(*args)
+
     def _write_oe(*args):
         """_write_oe(self, int which_dboard, int value, int mask) -> bool"""
         return _usrp1.usrp1_source_base__write_oe(*args)
@@ -431,57 +433,53 @@ class usrp1_source_base(object):
         """_read_spi(self, int optional_header, int enables, int format, int len) -> string"""
         return _usrp1.usrp1_source_base__read_spi(*args)
 
-
-class usrp1_source_basePtr(usrp1_source_base):
-    def __init__(self, this):
-        self.this = this
-        if not hasattr(self,"thisown"): self.thisown = 0
-        self.__class__ = usrp1_source_base
-_usrp1.usrp1_source_base_swigregister(usrp1_source_basePtr)
+usrp1_source_base_swigregister = _usrp1.usrp1_source_base_swigregister
+usrp1_source_base_swigregister(usrp1_source_base)
 
 def usrp1_source_base_make_format(*args):
-    """
+  """
     make_format(int width=16, int shift=0, bool want_q=True, bool bypass_halfband=False) -> unsigned int
     make_format(int width=16, int shift=0, bool want_q=True) -> unsigned int
     make_format(int width=16, int shift=0) -> unsigned int
     make_format(int width=16) -> unsigned int
     usrp1_source_base_make_format() -> unsigned int
     """
-    return _usrp1.usrp1_source_base_make_format(*args)
+  return _usrp1.usrp1_source_base_make_format(*args)
 
 def usrp1_source_base_format_width(*args):
-    """usrp1_source_base_format_width(unsigned int format) -> int"""
-    return _usrp1.usrp1_source_base_format_width(*args)
+  """usrp1_source_base_format_width(unsigned int format) -> int"""
+  return _usrp1.usrp1_source_base_format_width(*args)
 
 def usrp1_source_base_format_shift(*args):
-    """usrp1_source_base_format_shift(unsigned int format) -> int"""
-    return _usrp1.usrp1_source_base_format_shift(*args)
+  """usrp1_source_base_format_shift(unsigned int format) -> int"""
+  return _usrp1.usrp1_source_base_format_shift(*args)
 
 def usrp1_source_base_format_want_q(*args):
-    """usrp1_source_base_format_want_q(unsigned int format) -> bool"""
-    return _usrp1.usrp1_source_base_format_want_q(*args)
+  """usrp1_source_base_format_want_q(unsigned int format) -> bool"""
+  return _usrp1.usrp1_source_base_format_want_q(*args)
 
 def usrp1_source_base_format_bypass_halfband(*args):
-    """usrp1_source_base_format_bypass_halfband(unsigned int format) -> bool"""
-    return _usrp1.usrp1_source_base_format_bypass_halfband(*args)
+  """usrp1_source_base_format_bypass_halfband(unsigned int format) -> bool"""
+  return _usrp1.usrp1_source_base_format_bypass_halfband(*args)
 
 class usrp1_sink_c_sptr(object):
     """Proxy of C++ usrp1_sink_c_sptr class"""
-    def __repr__(self):
-        return "<%s.%s; proxy of C++ boost::shared_ptr<usrp1_sink_c > instance at %s>" % (self.__class__.__module__, self.__class__.__name__, self.this,)
-    def __init__(self, *args):
+    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
+    __repr__ = _swig_repr
+    def __init__(self, *args): 
         """
         __init__(self) -> usrp1_sink_c_sptr
         __init__(self,  p) -> usrp1_sink_c_sptr
         """
-        newobj = _usrp1.new_usrp1_sink_c_sptr(*args)
-        self.this = newobj.this
-        self.thisown = 1
-        del newobj.thisown
+        this = _usrp1.new_usrp1_sink_c_sptr(*args)
+        try: self.this.append(this)
+        except: self.this = this
     def __deref__(*args):
         """__deref__(self)"""
         return _usrp1.usrp1_sink_c_sptr___deref__(*args)
 
+    __swig_destroy__ = _usrp1.delete_usrp1_sink_c_sptr
+    __del__ = lambda self : None;
     def set_interp_rate(*args):
         """set_interp_rate(self, unsigned int rate) -> bool"""
         return _usrp1.usrp1_sink_c_sptr_set_interp_rate(*args)
@@ -566,6 +564,10 @@ class usrp1_sink_c_sptr(object):
         """set_adc_buffer_bypass(self, int which, bool bypass) -> bool"""
         return _usrp1.usrp1_sink_c_sptr_set_adc_buffer_bypass(*args)
 
+    def serial_number(*args):
+        """serial_number(self) -> string"""
+        return _usrp1.usrp1_sink_c_sptr_serial_number(*args)
+
     def _write_oe(*args):
         """_write_oe(self, int which_dboard, int value, int mask) -> bool"""
         return _usrp1.usrp1_sink_c_sptr__write_oe(*args)
@@ -674,51 +676,41 @@ class usrp1_sink_c_sptr(object):
         """set_detail(self, gr_block_detail_sptr detail)"""
         return _usrp1.usrp1_sink_c_sptr_set_detail(*args)
 
-    def __del__(self, destroy=_usrp1.delete_usrp1_sink_c_sptr):
-        """__del__(self)"""
-        try:
-            if self.thisown: destroy(self)
-        except: pass
-
-
-class usrp1_sink_c_sptrPtr(usrp1_sink_c_sptr):
-    def __init__(self, this):
-        self.this = this
-        if not hasattr(self,"thisown"): self.thisown = 0
-        self.__class__ = usrp1_sink_c_sptr
-_usrp1.usrp1_sink_c_sptr_swigregister(usrp1_sink_c_sptrPtr)
+usrp1_sink_c_sptr_swigregister = _usrp1.usrp1_sink_c_sptr_swigregister
+usrp1_sink_c_sptr_swigregister(usrp1_sink_c_sptr)
 
 
 def usrp1_sink_c_block(*args):
-    """usrp1_sink_c_block(usrp1_sink_c_sptr r) -> gr_block_sptr"""
-    return _usrp1.usrp1_sink_c_block(*args)
+  """usrp1_sink_c_block(usrp1_sink_c_sptr r) -> gr_block_sptr"""
+  return _usrp1.usrp1_sink_c_block(*args)
 usrp1_sink_c_sptr.block = lambda self: usrp1_sink_c_block (self)
 usrp1_sink_c_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id ())
 
 
 def sink_c(*args):
-    """
+  """
     sink_c(int which_board, unsigned int interp_rate, int nchan, 
-        int mux) -> usrp1_sink_c_sptr
+        int mux, string fpga_filename, string firmware_filename) -> usrp1_sink_c_sptr
     """
-    return _usrp1.sink_c(*args)
+  return _usrp1.sink_c(*args)
 class usrp1_sink_s_sptr(object):
     """Proxy of C++ usrp1_sink_s_sptr class"""
-    def __repr__(self):
-        return "<%s.%s; proxy of C++ boost::shared_ptr<usrp1_sink_s > instance at %s>" % (self.__class__.__module__, self.__class__.__name__, self.this,)
-    def __init__(self, *args):
+    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
+    __repr__ = _swig_repr
+    def __init__(self, *args): 
         """
         __init__(self) -> usrp1_sink_s_sptr
         __init__(self,  p) -> usrp1_sink_s_sptr
         """
-        newobj = _usrp1.new_usrp1_sink_s_sptr(*args)
-        self.this = newobj.this
-        self.thisown = 1
-        del newobj.thisown
+        this = _usrp1.new_usrp1_sink_s_sptr(*args)
+        try: self.this.append(this)
+        except: self.this = this
     def __deref__(*args):
         """__deref__(self)"""
         return _usrp1.usrp1_sink_s_sptr___deref__(*args)
 
+    __swig_destroy__ = _usrp1.delete_usrp1_sink_s_sptr
+    __del__ = lambda self : None;
     def set_interp_rate(*args):
         """set_interp_rate(self, unsigned int rate) -> bool"""
         return _usrp1.usrp1_sink_s_sptr_set_interp_rate(*args)
@@ -803,6 +795,10 @@ class usrp1_sink_s_sptr(object):
         """set_adc_buffer_bypass(self, int which, bool bypass) -> bool"""
         return _usrp1.usrp1_sink_s_sptr_set_adc_buffer_bypass(*args)
 
+    def serial_number(*args):
+        """serial_number(self) -> string"""
+        return _usrp1.usrp1_sink_s_sptr_serial_number(*args)
+
     def _write_oe(*args):
         """_write_oe(self, int which_dboard, int value, int mask) -> bool"""
         return _usrp1.usrp1_sink_s_sptr__write_oe(*args)
@@ -911,51 +907,41 @@ class usrp1_sink_s_sptr(object):
         """set_detail(self, gr_block_detail_sptr detail)"""
         return _usrp1.usrp1_sink_s_sptr_set_detail(*args)
 
-    def __del__(self, destroy=_usrp1.delete_usrp1_sink_s_sptr):
-        """__del__(self)"""
-        try:
-            if self.thisown: destroy(self)
-        except: pass
-
-
-class usrp1_sink_s_sptrPtr(usrp1_sink_s_sptr):
-    def __init__(self, this):
-        self.this = this
-        if not hasattr(self,"thisown"): self.thisown = 0
-        self.__class__ = usrp1_sink_s_sptr
-_usrp1.usrp1_sink_s_sptr_swigregister(usrp1_sink_s_sptrPtr)
+usrp1_sink_s_sptr_swigregister = _usrp1.usrp1_sink_s_sptr_swigregister
+usrp1_sink_s_sptr_swigregister(usrp1_sink_s_sptr)
 
 
 def usrp1_sink_s_block(*args):
-    """usrp1_sink_s_block(usrp1_sink_s_sptr r) -> gr_block_sptr"""
-    return _usrp1.usrp1_sink_s_block(*args)
+  """usrp1_sink_s_block(usrp1_sink_s_sptr r) -> gr_block_sptr"""
+  return _usrp1.usrp1_sink_s_block(*args)
 usrp1_sink_s_sptr.block = lambda self: usrp1_sink_s_block (self)
 usrp1_sink_s_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id ())
 
 
 def sink_s(*args):
-    """
+  """
     sink_s(int which_board, unsigned int interp_rate, int nchan, 
-        int mux) -> usrp1_sink_s_sptr
+        int mux, string fpga_filename, string firmware_filename) -> usrp1_sink_s_sptr
     """
-    return _usrp1.sink_s(*args)
+  return _usrp1.sink_s(*args)
 class usrp1_source_c_sptr(object):
     """Proxy of C++ usrp1_source_c_sptr class"""
-    def __repr__(self):
-        return "<%s.%s; proxy of C++ boost::shared_ptr<usrp1_source_c > instance at %s>" % (self.__class__.__module__, self.__class__.__name__, self.this,)
-    def __init__(self, *args):
+    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
+    __repr__ = _swig_repr
+    def __init__(self, *args): 
         """
         __init__(self) -> usrp1_source_c_sptr
         __init__(self,  p) -> usrp1_source_c_sptr
         """
-        newobj = _usrp1.new_usrp1_source_c_sptr(*args)
-        self.this = newobj.this
-        self.thisown = 1
-        del newobj.thisown
+        this = _usrp1.new_usrp1_source_c_sptr(*args)
+        try: self.this.append(this)
+        except: self.this = this
     def __deref__(*args):
         """__deref__(self)"""
         return _usrp1.usrp1_source_c_sptr___deref__(*args)
 
+    __swig_destroy__ = _usrp1.delete_usrp1_source_c_sptr
+    __del__ = lambda self : None;
     def set_decim_rate(*args):
         """set_decim_rate(self, unsigned int rate) -> bool"""
         return _usrp1.usrp1_source_c_sptr_set_decim_rate(*args)
@@ -1048,6 +1034,10 @@ class usrp1_source_c_sptr(object):
         """set_adc_buffer_bypass(self, int which, bool bypass) -> bool"""
         return _usrp1.usrp1_source_c_sptr_set_adc_buffer_bypass(*args)
 
+    def serial_number(*args):
+        """serial_number(self) -> string"""
+        return _usrp1.usrp1_source_c_sptr_serial_number(*args)
+
     def _write_oe(*args):
         """_write_oe(self, int which_dboard, int value, int mask) -> bool"""
         return _usrp1.usrp1_source_c_sptr__write_oe(*args)
@@ -1194,51 +1184,41 @@ class usrp1_source_c_sptr(object):
         """set_detail(self, gr_block_detail_sptr detail)"""
         return _usrp1.usrp1_source_c_sptr_set_detail(*args)
 
-    def __del__(self, destroy=_usrp1.delete_usrp1_source_c_sptr):
-        """__del__(self)"""
-        try:
-            if self.thisown: destroy(self)
-        except: pass
-
-
-class usrp1_source_c_sptrPtr(usrp1_source_c_sptr):
-    def __init__(self, this):
-        self.this = this
-        if not hasattr(self,"thisown"): self.thisown = 0
-        self.__class__ = usrp1_source_c_sptr
-_usrp1.usrp1_source_c_sptr_swigregister(usrp1_source_c_sptrPtr)
+usrp1_source_c_sptr_swigregister = _usrp1.usrp1_source_c_sptr_swigregister
+usrp1_source_c_sptr_swigregister(usrp1_source_c_sptr)
 
 
 def usrp1_source_c_block(*args):
-    """usrp1_source_c_block(usrp1_source_c_sptr r) -> gr_block_sptr"""
-    return _usrp1.usrp1_source_c_block(*args)
+  """usrp1_source_c_block(usrp1_source_c_sptr r) -> gr_block_sptr"""
+  return _usrp1.usrp1_source_c_block(*args)
 usrp1_source_c_sptr.block = lambda self: usrp1_source_c_block (self)
 usrp1_source_c_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id ())
 
 
 def source_c(*args):
-    """
+  """
     source_c(int which_board, unsigned int decim_rate, int nchan, 
-        int mux, int mode) -> usrp1_source_c_sptr
+        int mux, int mode, string fpga_filename, string firmware_filename) -> usrp1_source_c_sptr
     """
-    return _usrp1.source_c(*args)
+  return _usrp1.source_c(*args)
 class usrp1_source_s_sptr(object):
     """Proxy of C++ usrp1_source_s_sptr class"""
-    def __repr__(self):
-        return "<%s.%s; proxy of C++ boost::shared_ptr<usrp1_source_s > instance at %s>" % (self.__class__.__module__, self.__class__.__name__, self.this,)
-    def __init__(self, *args):
+    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
+    __repr__ = _swig_repr
+    def __init__(self, *args): 
         """
         __init__(self) -> usrp1_source_s_sptr
         __init__(self,  p) -> usrp1_source_s_sptr
         """
-        newobj = _usrp1.new_usrp1_source_s_sptr(*args)
-        self.this = newobj.this
-        self.thisown = 1
-        del newobj.thisown
+        this = _usrp1.new_usrp1_source_s_sptr(*args)
+        try: self.this.append(this)
+        except: self.this = this
     def __deref__(*args):
         """__deref__(self)"""
         return _usrp1.usrp1_source_s_sptr___deref__(*args)
 
+    __swig_destroy__ = _usrp1.delete_usrp1_source_s_sptr
+    __del__ = lambda self : None;
     def set_decim_rate(*args):
         """set_decim_rate(self, unsigned int rate) -> bool"""
         return _usrp1.usrp1_source_s_sptr_set_decim_rate(*args)
@@ -1331,6 +1311,10 @@ class usrp1_source_s_sptr(object):
         """set_adc_buffer_bypass(self, int which, bool bypass) -> bool"""
         return _usrp1.usrp1_source_s_sptr_set_adc_buffer_bypass(*args)
 
+    def serial_number(*args):
+        """serial_number(self) -> string"""
+        return _usrp1.usrp1_source_s_sptr_serial_number(*args)
+
     def _write_oe(*args):
         """_write_oe(self, int which_dboard, int value, int mask) -> bool"""
         return _usrp1.usrp1_source_s_sptr__write_oe(*args)
@@ -1477,33 +1461,22 @@ class usrp1_source_s_sptr(object):
         """set_detail(self, gr_block_detail_sptr detail)"""
         return _usrp1.usrp1_source_s_sptr_set_detail(*args)
 
-    def __del__(self, destroy=_usrp1.delete_usrp1_source_s_sptr):
-        """__del__(self)"""
-        try:
-            if self.thisown: destroy(self)
-        except: pass
-
-
-class usrp1_source_s_sptrPtr(usrp1_source_s_sptr):
-    def __init__(self, this):
-        self.this = this
-        if not hasattr(self,"thisown"): self.thisown = 0
-        self.__class__ = usrp1_source_s_sptr
-_usrp1.usrp1_source_s_sptr_swigregister(usrp1_source_s_sptrPtr)
+usrp1_source_s_sptr_swigregister = _usrp1.usrp1_source_s_sptr_swigregister
+usrp1_source_s_sptr_swigregister(usrp1_source_s_sptr)
 
 
 def usrp1_source_s_block(*args):
-    """usrp1_source_s_block(usrp1_source_s_sptr r) -> gr_block_sptr"""
-    return _usrp1.usrp1_source_s_block(*args)
+  """usrp1_source_s_block(usrp1_source_s_sptr r) -> gr_block_sptr"""
+  return _usrp1.usrp1_source_s_block(*args)
 usrp1_source_s_sptr.block = lambda self: usrp1_source_s_block (self)
 usrp1_source_s_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id ())
 
 
 def source_s(*args):
-    """
+  """
     source_s(int which_board, unsigned int decim_rate, int nchan, 
-        int mux, int mode) -> usrp1_source_s_sptr
+        int mux, int mode, string fpga_filename, string firmware_filename) -> usrp1_source_s_sptr
     """
-    return _usrp1.source_s(*args)
+  return _usrp1.source_s(*args)
 
 
