@@ -1,6 +1,6 @@
 /* -*- c++ -*- */
 /*
- * Copyright 2004 Free Software Foundation, Inc.
+ * Copyright 2004,2006 Free Software Foundation, Inc.
  * 
  * This file is part of GNU Radio
  * 
@@ -43,7 +43,9 @@ class usrp1_sink_base : public gr_sync_block {
 		   int which_board,
 		   unsigned int interp_rate,
 		   int nchan,
-		   int mux
+		   int mux,
+		   const std::string fpga_filename,
+		   const std::string firmware_filename
 		   ) throw (std::runtime_error);
 
   /*!
@@ -242,6 +244,13 @@ class usrp1_sink_base : public gr_sync_block {
    */
   bool set_adc_buffer_bypass (int which, bool bypass);
 
+  /*!
+   * \brief return the usrp's serial number.
+   *
+   * \returns non-zero length string iff successful.
+   */
+  std::string serial_number();
+
   /*!
    * \brief Write direction register (output enables) for pins that go to daughterboard.
    *
