@@ -37,7 +37,9 @@ usrp1_source_base::usrp1_source_base (const std::string &name,
 				      unsigned int decim_rate,
 				      int nchan,
 				      int mux,
-				      int mode
+				      int mode,
+				      const std::string fpga_filename,
+				      const std::string firmware_filename
 				      ) throw (std::runtime_error)
   : gr_sync_block (name,
 		   gr_make_io_signature (0, 0, 0),
@@ -45,7 +47,11 @@ usrp1_source_base::usrp1_source_base (const std::string &name,
     d_noverruns (0)
 {
   d_usrp = usrp_standard_rx::make (which_board, decim_rate,
-				   nchan, mux, mode);
+				   nchan, mux, mode,
+				   0,			// fusb_block_size
+				   0,			// fusb_nblocks
+				   fpga_filename,
+				   firmware_filename);
   if (d_usrp == 0)
     throw std::runtime_error ("can't open usrp1");
 
@@ -302,6 +308,12 @@ usrp1_source_base::set_adc_buffer_bypass (int which, bool bypass)
   return d_usrp->set_adc_buffer_bypass (which, bypass);
 }
 
+std::string
+usrp1_source_base::serial_number()
+{
+  return d_usrp->serial_number();
+}
+
 bool
 usrp1_source_base::_write_oe (int which_dboard, int value, int mask)
 {
