@@ -38,9 +38,11 @@ typedef boost::shared_ptr<usrp1_source_s> usrp1_source_s_sptr;
 usrp1_source_s_sptr
 usrp1_make_source_s (int which_board, 
 		     unsigned int decim_rate,
-		     int nchan = 1,
-		     int mux = 0x32103210,
-		     int mode = 0
+		     int nchan,
+		     int mux,
+		     int mode,
+		     const std::string fpga_filename,
+		     const std::string firmware_filename
 		     ) throw (std::runtime_error);
 
 /*!
@@ -55,15 +57,19 @@ class usrp1_source_s : public usrp1_source_base {
 		       unsigned int decim_rate,
 		       int nchan,
 		       int mux,
-		       int mode
+		       int mode,
+		       const std::string fpga_filename,
+		       const std::string firmware_filename
 		       ) throw (std::runtime_error);
 
  protected:
   usrp1_source_s (int which_board, 
 		  unsigned int decim_rate,
-		  int nchan = 1,
-		  int mux = 0x32103210,
-		  int mode = 0
+		  int nchan,
+		  int mux,
+		  int mode,
+		  const std::string fpga_filename,
+		  const std::string firmware_filename
 		  ) throw (std::runtime_error);
 
   virtual int ninput_bytes_reqd_for_noutput_items (int noutput_items);
